@@ -13,9 +13,14 @@ def is_windows() -> bool:
     return platform.system() == "Windows"
 
 
+_ABSEIL_REQ = "abseil-cpp==20220623.*"
+_BENCHMARK_REQ = "benchmark>=1.7,<1.8"
 _CMAKE_REQ = "cmake>=3.18"
 _NUMPY_REQ = "numpy>=1.22,<1.24"
-_TBB_DEVEL_REQ = "tbb-devel==2021.6.*"
+_TBB_VER = "==2021.6.*"
+_TBB_REQ = "tbb" + _TBB_VER
+_TBB_DEVEL_REQ = "tbb-devel" + _TBB_VER
+_ZSTD_REQ = "zstd>=1.5.2,<1.6"
 
 # Host toolchain requirements to build riptide_cpp.
 toolchain_reqs = []
@@ -36,37 +41,51 @@ conda_reqs = [
 ] + toolchain_reqs
 
 # PyPI setup build requirements.
-# Most everything else will be specified in setup.py.
+# Most everything else *should* be in pyproject.toml, but since we run
+# setup.py directly we need to set up build environment manually here.
 pypi_reqs = [
-    _CMAKE_REQ,  # TODO: Remove this once setup.py sdist properly requires cmake!
+    _ABSEIL_REQ,  # PyPI package doesn't exist
+    _BENCHMARK_REQ,  # PyPI package doesn't exist
+    _CMAKE_REQ,
     _TBB_DEVEL_REQ,  # needed because PyPI tbb-devel pkg doesn't contain CMake files yet
     "wheel",
+    _ZSTD_REQ,  # PyPI package doesn't exist
 ] + toolchain_reqs
 
 # Native CMake build requirements.
 native_reqs = [
+    _ABSEIL_REQ,
+    _BENCHMARK_REQ,
     _CMAKE_REQ,
     _NUMPY_REQ,
     _TBB_DEVEL_REQ,
+    _ZSTD_REQ,
 ] + toolchain_reqs
 
 # Extras build requirements (same as native build requirements).
 extras_reqs = native_reqs
 
-# Runtime requirements for riptable and riptide_cpp.
+# Runtime requirements for riptide_cpp.
 # Replicates runtime requirements in meta.yaml and setup.py.
 runtime_reqs = [
-    "ansi2html>=1.5.2",
-    "numba>=0.56.2",
+    # Disable abseil-cpp requirements to avoid pip install failures (no PyPI package exists)
+    # Assume it's conda install'ed as part of build-level requirements.
+    # _ABSEIL_REQ,
     _NUMPY_REQ,
-    "pandas>=0.24,<2.0",
-    "python-dateutil",
-    "tbb==2021.6.*",
+    _TBB_REQ,
+    _ZSTD_REQ,
 ]
 
-# Complete test requirements for riptable tests.
-# Needed to engage all features and their tests.
+# Complete test requirements for riptide_cpp tests.
+# Needed to engage all features and their tests (including riptable/python tests).
 tests_reqs = [
+    # Disable benchmark requirement to avoid pip install failures (PyPI pkg is 'google-benchmark')
+    # Assume it's conda install'ed as part of build-level requirements.
+    # TODO: Can extend this script to support PyPI vs Conda requirements to map correct names
+    # _BENCHMARK_REQ,
+]
+# Add riptable tests requirements
+tests_reqs += [
     "arrow",
     "bokeh",
     "bottleneck",
@@ -74,21 +93,38 @@ tests_reqs = [
     "hypothesis",
     "ipykernel",
     "ipython",
+    "matplotlib",
     "nose",
     "pyarrow",
     "pytest",
 ]
+# Add riptable runtime requirements
+tests_reqs += [
+    "ansi2html>=1.5.2",
+    "numba>=0.56.2",
+    _NUMPY_REQ,
+    "pandas>=0.24,<2.0",
+    "python-dateutil",
+]
 
 # Complete developer requirements.
 # Union of all above requuirements plus those needed for code contributions.
 developer_reqs = (
-    ["clang-format", "setuptools_scm"]
+    [
+        "black",
+        "clang-format",
+        "setuptools_scm",
+    ]
     + conda_reqs
     + pypi_reqs
     + runtime_reqs
     + tests_reqs
     + toolchain_reqs
 )
+if is_linux():
+    developer_reqs += [
+        "gdb",
+    ]
 
 target_reqs = {
     "conda": conda_reqs,
